// Copyright (C) 2008
// by all contributors.

// This file is part of SCNSL.

// SCNSL is free software:
// you can redistribute it and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option)
// any later version.

// SCNSL is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY;
// without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.
// See the GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with SCNSL,
// in a file named LICENSE.txt.
// If not, see <http://www.gnu.org/licenses/>.



/// @file
///


#include "scnsl/protocols/router_communicator//RouterCommunicator_t.hh"

using namespace Scnsl::Protocols::RouterCommunicator;

// ////////////////////////////////////////////////////////////////
// Constructor and destructor.
// ////////////////////////////////////////////////////////////////


RouterCommunicator_t::RouterCommunicator_t(
    const RoutingTable_t & routingTable, Scnsl::Core::Node_t * rn ):
    // Parents:
    Scnsl::Core::Communicator_if_t(),
    // Fields:
    _routerNode( rn ),
    _routingTable( routingTable ),
    _is_sending( false )
{
    // Nothing to do.
}



RouterCommunicator_t::~RouterCommunicator_t()
{
    // Nothing to do.
}


// ////////////////////////////////////////////////////////////////
// Interface methods.
// ////////////////////////////////////////////////////////////////



RouterCommunicator_t::errorcode_t RouterCommunicator_t::send( const Scnsl::Core::Packet_t & p)
    
{
    if ( _is_sending ) return static_cast<errorcode_t>(-1);
    _is_sending = true;

	RoutingTable_t::iterator i = _routingTable.find(
        const_cast< Scnsl::Core::Node_t * >( p.getDestinationNode() ));
    if ( i == _routingTable.end() )
		throw std::logic_error( "Error: node not found in routingTable" );

	std::pair <Scnsl::Core::Channel_if_t * ,Scnsl::Core::Node_t * > values = i->second;

	Scnsl::Core::Channel_if_t * channel = values.first;
	Scnsl::Core::Node_t * destinationNode = values.second;

	if (( channel == NULL) || ( destinationNode == NULL ))
    {
		throw std::logic_error( "Error: associated channel or destinationNode not found in the routing table" );
	}

    Scnsl::Core::Packet_t pRef(p);
    pRef.setChannel(channel);
	Scnsl::Core::Packet_t pRouter;
	pRouter.setPayload(pRef);
	pRouter.setSourceTaskProxy( pRef.getSourceTaskProxy() );
	pRouter.setChannel(channel);
	pRouter.setDestinationNode(destinationNode);
	pRouter.setId( pRef.getId() );
	pRouter.setLabel( pRef.getLabel() );

	errorcode_t ret = Communicator_if_t::send(pRouter);
    _is_sending = false;
    return ret;
}


RouterCommunicator_t::errorcode_t RouterCommunicator_t::receive( const Scnsl::Core::Packet_t & p)
    
{
	Scnsl::Core::Packet_t pRouter;
	p.getPayload(pRouter);

	if(pRouter.getDestinationNode()==_routerNode)
	{
		return Communicator_if_t::receive(pRouter);
	}
	else
	{
		return this->send(pRouter);
	}

}


