/// @file
/// A TLM task.

#include <sstream>
#include <stdexcept>

#include "MyTask2.hh"



// ////////////////////////////////////////////////////////////////
// Constructor.
// ////////////////////////////////////////////////////////////////

MyTask2::MyTask2( sc_core::sc_module_name modulename,
                  const task_id_t id,
                  Scnsl::Core::Node_t * n,
                  const size_t proxies,
                  const int scenario )
    :
    // Parents:
    Scnsl::Tlm::TlmTask_if_t( modulename, id, n, proxies )
{

	switch ( scenario )
	{
		case 1:
            break;
		case 2: 	SC_THREAD( writingProcess2 );
            break;
		case 3: 	SC_THREAD( writingProcess3 );
            break;
		case 4: 	SC_THREAD( writingProcess4 );
            break;
		default:    throw  std::invalid_argument( "scenario parameter valid values are from 1 to 4" );
	}

}


MyTask2::~MyTask2()  {}


void MyTask2::writingProcess2()
{
	wait( 34, sc_core::SC_MS );

	for ( int j = 0; j < 2; j++ )
	{
		byte_t i = byte_t( rand()%25 + 65 );
	    const std::string tp = "0";
		const size_t s = 1;

		std::cout << "Time: " << sc_core::sc_time_stamp() << ", Name: " << name() << ", data sended: " << i;
		std::cout << ", packet size: " << s << " bytes" << std::endl;

		TlmTask_if_t::send( tp, &i, s);
	}
}


void MyTask2::writingProcess3()
{

	wait( 18, sc_core::SC_MS );

	for ( ;; )
	{
		byte_t i = byte_t( rand()%25 + 65 );
	    const std::string tp = "0";
		const size_t s = 1;

		std::cout << "Time: " << sc_core::sc_time_stamp() << ", Name: " << name() << ", data sended: " << i;
		std::cout << ", packet size: " << s << " bytes" << std::endl;

		TlmTask_if_t::send( tp, &i, s);

		wait(28, sc_core::SC_MS);
	}
}


void MyTask2::writingProcess4()
{
	wait( 16, sc_core::SC_MS );
	wait( 2, sc_core::SC_MS );

	for ( ;; )
	{
		byte_t i = byte_t( rand()%25 + 65 );
	    const std::string tp = "0";
		const size_t s = 1;

		std::cout << "Time: " << sc_core::sc_time_stamp() << ", Name: " << name() << ", data sended: " << i;
		std::cout << ", packet size: " << s << " bytes" << std::endl;

		TlmTask_if_t::send( tp, &i, s);

		wait( 20, sc_core::SC_MS );
		wait( 8, sc_core::SC_MS );

	}
}


void MyTask2::b_transport( tlm::tlm_generic_payload & p, sc_core::sc_time & t ){

	t=t;
	char result;
	bool c;

	if(p.get_command() == Scnsl::Tlm::tlm_command_t(Scnsl::Tlm::PACKET_COMMAND))
	{
    	result = static_cast< char >( * p.get_data_ptr() );
		std::cout << "Time: " << sc_core::sc_time_stamp() << ", Name: " << name() << ", data received: " << result;
		std::cout << ", packet size: " << p.get_data_length() << " bytes" << std::endl;
	}

	if(p.get_command() == Scnsl::Tlm::tlm_command_t(Scnsl::Tlm::CARRIER_COMMAND))
	{
		c = *p.get_data_ptr() != 0;
		std::cout << "Time: " << sc_core::sc_time_stamp() << ", Name: " << name() << ", carrier: " << c << std::endl;
	}

}



