
#include <sstream>
#include <systemc>
#include <tlm.h>
#include <exception>

#include <scnsl.hh>
#include "MyTask.hh"


using namespace Scnsl;
using namespace Scnsl::Core;
using namespace Scnsl::BuiltinPlugin;
using namespace Scnsl::Setup;
using namespace std;
using namespace Scnsl::Tlm;


int sc_main( int argc, char * argv[] )
{
	try {

        // Singleton.
        Scnsl::Setup::Scnsl_t * sim = Scnsl::Setup::Scnsl_t::get_instance();

        // Nodes creation:
        Scnsl::Core::Node_t * n1 = sim->createNode();
        Scnsl::Core::Node_t * n2 = sim->createNode();


		CoreCommunicatorSetup_t ccs;
		ccs.extensionId = "core";
		ccs.type = CoreCommunicatorSetup_t::QUEUE;
		ccs.queueSend = CoreCommunicatorSetup_t::FIFO;
		ccs.queueReceive = CoreCommunicatorSetup_t::FIFO;
		ccs.capacitySend = 5;
		ccs.capacityReceive = 5;
		ccs.name = "ccs_fifo";

		Communicator_if_t * coda = sim->createCommunicator( ccs );



        CoreChannelSetup_t csb;

        csb.channel_type = CoreChannelSetup_t::FULL_DUPLEX;
        csb.capacity = 1000;
		csb.capacity2 = 1000;
        csb.delay = sc_core::sc_time(10.0, sc_core::SC_MS);
        csb.extensionId = "core";
        csb.name = "channel_fullduplex";

        Scnsl::Core::Channel_if_t * ch1 = sim->createChannel( csb );

        BindSetup_base_t bsb1;
        BindSetup_base_t bsb2;
        bsb1.extensionId = "core";
        bsb2.extensionId = "core";
        sim->bind( n1, ch1, bsb1 );
        sim->bind( n2, ch1, bsb2 );

        MyTask t1 ( "task1", 1, n1, 1 );
        MyTask t2 ( "task2", 2, n2, 1 );

        sim->bind( & t1, NULL, ch1, bsb1, coda );
        sim->bind( & t2, NULL, ch1, bsb2, NULL );

        sc_core::sc_start( sc_core::sc_time( 100, sc_core::SC_MS ) );
        sc_core::sc_stop();

	}
    catch (exception & e)
    {
		cout << e.what() << endl;

        cout << argc << endl;
        cout << argv[0] << endl;
	}
	return 0;
}
